"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
// Modules
const events_1 = require("events");
const Websocket_1 = require("./Websocket");
const index_1 = __importDefault(require("./Rest/index"));
// Collections
const House_1 = require("./Collections/House");
const Message_1 = require("./Collections/Message");
const Member_1 = require("./Collections/Member");
const Room_1 = require("./Collections/Room");
const User_1 = require("./Collections/User");
class Client extends events_1.EventEmitter {
    constructor(options = { type: 'bot' }) {
        super();
        this.options = options;
        this.ws = new Websocket_1.WS();
        exports.ws = this.ws;
        this.rest = new index_1.default(this);
        exports.rest = this.rest;
        exports.client = this;
        this.users = new User_1.User(this);
        this.rooms = new Room_1.Room(this);
        this.houses = new House_1.House(this);
        this.members = new Member_1.Member(this);
        this.messages = new Message_1.Message(this);
    }
    connect(token) {
        return __awaiter(this, void 0, void 0, function* () {
            // Set the token in the instance
            if (this.options.type === 'bot')
                this.token = `Bot ${token}`;
            else
                this.token = token;
            this.rest.init(this);
            // Connect to the socket
            yield this.ws.init();
            // Auth with the socket
            yield this.ws.sendOp(2, { token: this.token });
            this.ws.on('data', (body) => {
                var _a, _b, _c, _d;
                const { e, d } = body;
                // Emits every event to RAW
                const raw = {
                    event: e,
                    data: d
                };
                this.emit('RAW', raw);
                switch (e) {
                    case 'INIT_STATE': {
                        this.user = d.user;
                        this.users.collect(d.user.id, d.user);
                        d.private_rooms.forEach((room) => {
                            var _a;
                            const house = this.houses.resolve(d.house_id);
                            const collectRoom = {
                                id: room.id,
                                name: room.name,
                                house: house,
                                position: room.position,
                                type: room.type,
                                description: room.description,
                                permission_overwrites: room.permission_overwrites,
                                recipients: new User_1.User(exports.client),
                                typing: room.typing,
                                last_message_id: room.last_message_id,
                                emoji: room.emoji,
                                messages: new Message_1.Message(exports.client)
                            };
                            (_a = room.recipients) === null || _a === void 0 ? void 0 : _a.forEach((recipient) => {
                                this.users.collect(recipient.id, recipient);
                                collectRoom.recipients.collect(recipient.id, recipient);
                            });
                            this.rooms.collect(room.id, collectRoom);
                        });
                        this.emit('init', d);
                        return this.emit(e, d);
                    }
                    case 'ROOM_CREATE': {
                        const createRoomHouse = this.houses.resolve(d.house_id);
                        const finalRoom = {
                            id: d.id,
                            name: d.name,
                            house: createRoomHouse,
                            position: d.position,
                            type: d.type,
                            description: d.description,
                            permission_overwrites: d.permission_overwrites,
                            recipients: d.recipients,
                            typing: d.typing,
                            last_message_id: d.last_message_id,
                            emoji: d.emoji,
                            messages: new Message_1.Message(exports.client)
                        };
                        this.rooms.collect(d.id, finalRoom);
                        if (createRoomHouse) {
                            (_a = createRoomHouse === null || createRoomHouse === void 0 ? void 0 : createRoomHouse.rooms) === null || _a === void 0 ? void 0 : _a.collect(d.id, finalRoom);
                            this.houses.collect((createRoomHouse === null || createRoomHouse === void 0 ? void 0 : createRoomHouse.id) || '', createRoomHouse);
                        }
                        this.emit('room_create', finalRoom);
                        return this.emit(e, d);
                    }
                    case 'ROOM_UPDATE': {
                        this.emit('room_update', d);
                        return this.emit(e, d);
                    }
                    case 'ROOM_DELETE': {
                        const rooms = (_b = this.houses.resolve(d.house_id).rooms) === null || _b === void 0 ? void 0 : _b.resolve(d.id);
                        this.houses.resolve(d.house_id).rooms = rooms;
                        this.rooms.delete(d.id);
                        this.emit('room_delete', d);
                        return this.emit(e, d);
                    }
                    case 'MESSAGE_CREATE': {
                        try {
                            const house = this.houses.resolve(d.house_id);
                            const room = this.rooms.resolve(d.room_id);
                            const message = {
                                id: d.id,
                                room,
                                content: d.content,
                                timestamp: new Date(d.timestamp),
                                house,
                                author: this.users.resolve(d.author_id),
                                member: (_c = house === null || house === void 0 ? void 0 : house.members) === null || _c === void 0 ? void 0 : _c.resolve(d.author_id)
                            };
                            if (room)
                                room.messages.collect(message.id, message);
                            this.messages.collect(d.id, message);
                            this.emit('message', message);
                            return this.emit(e, message);
                        }
                        catch (error) {
                            console.log(error);
                        }
                    }
                    case 'MESSAGE_UPDATE': {
                        this.emit('message_update', d);
                        return this.emit(e, d);
                    }
                    case 'MESSAGE_DELETE': {
                        this.messages.delete(d.id);
                        this.emit('message_delete', d);
                        return this.emit(e, d);
                    }
                    case 'TYPING_START': {
                        this.emit('typing_start', d);
                        return this.emit(e, d);
                    }
                    case 'HOUSE_JOIN': {
                        const joinHouse = d;
                        let house = this.houses.resolve(joinHouse.id);
                        if (!house)
                            house = this.houses.collect(joinHouse.id, {
                                id: joinHouse.id,
                                name: joinHouse.name,
                                members: new Member_1.Member(exports.client),
                                rooms: new Room_1.Room(exports.client)
                            });
                        d.rooms.forEach((room) => __awaiter(this, void 0, void 0, function* () {
                            var _e;
                            const finalRoom = {
                                id: room.id,
                                name: room.name,
                                house,
                                position: room.position,
                                type: room.type,
                                description: room.description,
                                permission_overwrites: room.permission_overwrites,
                                recipients: room.recipients,
                                typing: room.typing,
                                last_message_id: room.last_message_id,
                                emoji: room.emoji,
                                messages: new Message_1.Message(exports.client)
                            };
                            this.rooms.collect(room.id, finalRoom);
                            (_e = house.rooms) === null || _e === void 0 ? void 0 : _e.collect(room.id, finalRoom);
                        }));
                        d.members.forEach((member) => __awaiter(this, void 0, void 0, function* () {
                            var _f;
                            (_f = house.members) === null || _f === void 0 ? void 0 : _f.collect(member.id, {
                                id: member.id,
                                roles: member.roles,
                                last_message_id: member.last_message_id,
                                house,
                                user: this.users.resolve(member.id)
                            });
                            this.users.collect(member.id, member.user);
                            this.members.collect(member.id, member.user);
                        }));
                        // Cache houses
                        this.houses.collect(house.id || '', {
                            id: house.id,
                            name: house.name,
                            owner: this.users.resolve(joinHouse.owner_id) || joinHouse.owner_id,
                            icon: house.icon,
                            members: house.members,
                            rooms: house.rooms,
                            banner: house.banner,
                            synced: house.synced
                        });
                        this.emit('house_join', house);
                        return this.emit(e, house);
                    }
                    case 'HOUSE_MEMBER_JOIN': {
                        const houseJoin = this.houses.resolve(d.house_id);
                        const member = d;
                        (_d = houseJoin.members) === null || _d === void 0 ? void 0 : _d.collect(member.id, {
                            id: member.id,
                            house: houseJoin
                        });
                        this.houses.collect(houseJoin.id || '', houseJoin);
                        this.users.collect(d.user_id, d.user);
                        this.emit('house_member_join', d);
                        return this.emit(e, member);
                    }
                    case 'HOUSE_MEMBER_LEAVE': {
                        // Soon
                        this.emit('house_member_leave', d);
                        return this.emit(e, d);
                    }
                    case 'CALL_CREATE': {
                        const room = this.rooms.resolve(d.room_id);
                        room.join_token = d.join_token;
                        this.emit('call_create', d);
                        return this.emit(e, d);
                    }
                    default: {
                        break;
                    }
                }
                if (body && d)
                    this.emit(e, d);
            });
        });
    }
    destroy() {
        this.ws.destroy();
    }
}
exports.Client = Client;
//# sourceMappingURL=Client.js.map