"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.House = void 0;
// Collection Base
const BaseCollection_1 = require("./BaseCollection");
const Client_1 = require("../Client");
const Snowflake_1 = require("../Utils/Snowflake");
// House class
class House extends BaseCollection_1.BaseCollection {
    constructor(client) {
        super();
        this.client = client;
    }
    collect(key, value) {
        if (typeof value == 'object') {
            if (value.id && Snowflake_1.SnowflakeToDate(value.id))
                value.created = Snowflake_1.SnowflakeToDate(value.id);
            value.createInvite = this.createInvite;
            value.leave = this.leave;
            value.createRoom = this.createRoom;
        }
        super.set(key, value);
        return super.get(key);
    }
    create(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const createHouse = yield Client_1.rest.post('/houses', {
                data: { name, icon: null }
            });
            return createHouse;
        });
    }
    createInvite(uses, age) {
        return __awaiter(this, void 0, void 0, function* () {
            const createInvite = yield Client_1.rest.post(`/houses/${this.id}/invites`, {
                data: { max_uses: uses, max_age: age }
            });
            return createInvite.data.data;
        });
    }
    createRoom(name) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!name)
                throw 'missing_name';
            // Create Room
            const createRoom = yield Client_1.rest.post(`/houses/${this.id}/rooms`, {
                data: { name }
            });
            if (!createRoom)
                throw 'failed_to_create_room';
            (_a = this.rooms) === null || _a === void 0 ? void 0 : _a.collect(createRoom.id, {
                id: createRoom.id,
                name: createRoom.name,
                house: this,
                position: createRoom.position,
                type: createRoom.type
            });
            return yield ((_b = this.client) === null || _b === void 0 ? void 0 : _b.rooms.collect(createRoom.id, {
                id: createRoom.id,
                name: createRoom.name,
                house: this,
                position: createRoom.position,
                type: createRoom.type
            }));
        });
    }
    Join(code) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const getInvite = yield Client_1.rest.get(`/invites/${code}`);
            if (!getInvite.data)
                throw 'failed_to_fetch_invite';
            const useInvite = yield Client_1.rest.post(`/invites/${code}`, {});
            if (!useInvite.data)
                throw 'failed_to_use_invite';
            // Fetch the house from the house store by the ID and return that with the callback
            return yield this.collect(getInvite.data.data.house.id, {
                id: getInvite.data.data.house.id,
                name: getInvite.data.data.house.name,
                owner: (_a = this.client) === null || _a === void 0 ? void 0 : _a.users.resolve(getInvite.data.data.house.owner_id),
                icon: getInvite.data.data.house.icon,
                members: getInvite.data.data.house.members,
                rooms: getInvite.data.data.house.rooms
            });
        });
    }
    leave() {
        return __awaiter(this, void 0, void 0, function* () {
            // Leave the house
            const leaveHouse = yield Client_1.rest.delete(`/users/@me/houses/${this.id}`);
            return leaveHouse;
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            const deleteHouse = yield Client_1.rest.delete(`/houses/${this.id}`);
            return deleteHouse;
        });
    }
}
exports.House = House;
//# sourceMappingURL=House.js.map