"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Room = void 0;
// Collection Base
const BaseCollection_1 = require("./BaseCollection");
const Client_1 = require("../Client");
const Voice_1 = require("../Voice");
const Snowflake_1 = require("../Utils/Snowflake");
class Room extends BaseCollection_1.BaseCollection {
    constructor(client) {
        super();
        this.create = (name) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            if (!this.id)
                throw 'cannot_call_without_house';
            if (!name)
                throw 'missing_name';
            // Create Room
            const createRoom = yield Client_1.rest.post(`/houses/${this.id}/rooms`, {
                data: { name }
            });
            if (!createRoom)
                throw 'failed_to_create_room';
            (_b = (_a = this.house) === null || _a === void 0 ? void 0 : _a.rooms) === null || _b === void 0 ? void 0 : _b.collect(createRoom.id, {
                id: createRoom.id,
                name: createRoom.name,
                house: this,
                position: createRoom.position,
                type: createRoom.type
            });
            return yield this.client.rooms.collect(createRoom.id, {
                id: createRoom.id,
                name: createRoom.name,
                house: this,
                position: createRoom.position,
                type: createRoom.type
            });
        });
        this.client = client;
    }
    /**
     *
     * @param {string} key Object key
     * @param {value} value Object value
     */
    collect(key, value) {
        if (typeof value == 'object') {
            if (value.id && Snowflake_1.SnowflakeToDate(value.id))
                value.created = Snowflake_1.SnowflakeToDate(value.id);
            value.client = this.client;
            value.send = this.send;
            value.delete = this.delete;
            value.join = this.join;
            value.leave = this.leave;
            value.startTyping = this.startTyping;
        }
        super.set(key, value);
        return super.get(key);
    }
    /**
     * Send function to send message to room
     * @param {string} content Message contents
     */
    send(content) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const sendMessage = yield Client_1.rest.post(`/rooms/${this.id}/messages`, {
                data: { content }
            });
            const message = {
                id: sendMessage.data.id,
                content: sendMessage.data.content,
                timestamp: new Date(sendMessage.data.timestamp),
                room: (_a = this.client) === null || _a === void 0 ? void 0 : _a.rooms.get(sendMessage.data.room_id),
                house: (_b = this.client) === null || _b === void 0 ? void 0 : _b.houses.get(sendMessage.data.house_id),
                author: (_c = this.client) === null || _c === void 0 ? void 0 : _c.users.get(sendMessage.data.author_id)
            };
            const collect = (_d = this.client) === null || _d === void 0 ? void 0 : _d.messages.collect(message.id, message);
            return collect;
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield Client_1.rest.delete(`/houses/${this.id}/rooms/${this.id}`);
        });
    }
    join() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: Write logic to create call if there is not a call going yet
            // await rest.post<void>(`/rooms/${this.id}/call`);
            // Obtain the voice join token
            yield Client_1.ws.sendOp(4, { id: this.id });
            let timeout;
            yield Promise.race([
                new Promise((resolve) => (timeout = setTimeout(resolve, 5000))),
                new Promise((resolve) => {
                    this.client.once('CALL_CREATE', resolve);
                    clearTimeout(timeout);
                })
            ]);
            // Create voice handler and init the connection
            const voice = new Voice_1.Voice();
            console.log(this.client.user, this.join_token);
            voice.init(this.id, ((_b = (_a = this.client) === null || _a === void 0 ? void 0 : _a.user) === null || _b === void 0 ? void 0 : _b.id) || '', this.join_token || '');
            voice.on('data', (data) => {
                console.log('RAW Voice Data', JSON.stringify(data));
            });
            voice.on('connection', (data) => {
                console.log('Connection created', data);
                Client_1.ws.sendOp(5, { room_id: this.id, muted: false });
            });
            return;
        });
    }
    leave() {
        Client_1.ws.sendOp(5, { room_id: null, muted: false });
    }
    /**
     * Start typing in the room for 10 seconds
     */
    startTyping() {
        return __awaiter(this, void 0, void 0, function* () {
            yield Client_1.rest.post(`/rooms/${this.id}/typing`, {});
        });
    }
}
exports.Room = Room;
//# sourceMappingURL=Room.js.map