"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.User = void 0;
// Collection Base
const BaseCollection_1 = require("./BaseCollection");
const Client_1 = require("../Client");
const Snowflake_1 = require("../Utils/Snowflake");
class User extends BaseCollection_1.BaseCollection {
    constructor(client) {
        super();
        this.collect = (key, value) => {
            // If the value has an id add a .created attribute
            if (typeof value == 'object' && value.id && Snowflake_1.SnowflakeToDate(value.id))
                value.created = Snowflake_1.SnowflakeToDate(value.id);
            return super.set(key, value);
        };
        this.destroy = (key) => {
            return super.delete(key);
        };
        this.client = client;
    }
    send(content, id) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const sendMessage = yield Client_1.rest.post(`/rooms/${id}/messages`, {
                data: { content }
            });
            const message = {
                id: sendMessage.data.id,
                content: sendMessage.data.content,
                timestamp: new Date(sendMessage.data.timestamp),
                room: (_a = this.client) === null || _a === void 0 ? void 0 : _a.rooms.get(sendMessage.data.room_id),
                house: (_b = this.client) === null || _b === void 0 ? void 0 : _b.houses.get(sendMessage.data.house_id),
                author: (_c = this.client) === null || _c === void 0 ? void 0 : _c.users.get(sendMessage.data.author_id)
            };
            const collect = yield ((_d = this.client) === null || _d === void 0 ? void 0 : _d.messages.collect(message.id, message));
            return collect;
        });
    }
}
exports.User = User;
//# sourceMappingURL=User.js.map