"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// Modules
const node_fetch_1 = __importDefault(require("node-fetch"));
const ApiError_1 = __importDefault(require("../Errors/ApiError"));
// Constants
const API_VERSION = 'v1';
const API_URL = `${process.env.REST_HOST || 'https://api.hiven.io'}/${API_VERSION}`;
const USER_AGENT = 'Hiven.js (Version 1.0.0) - https://github.com/hivenapp/hiven.js';
// Rest class
class Rest {
    constructor(client) {
        this.client = client;
    }
    init(client) {
        return __awaiter(this, void 0, void 0, function* () {
            this.client = client;
            this.API_HEADERS = {
                authorization: this.client.token,
                'user-agent': USER_AGENT
            };
        });
    }
    build({ method, path, data, headers }) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const url = `${API_URL}${path}`;
                headers = { 'content-type': 'text/plain' };
                if (data) {
                    data = JSON.stringify(data);
                    headers['content-type'] = 'application/json';
                }
                else
                    headers['content-type'] = '';
                const res = yield node_fetch_1.default(url, {
                    method,
                    body: method != 'get' ? data : null,
                    headers: Object.assign(Object.assign({}, headers), this.API_HEADERS)
                });
                let body;
                if ((_a = res.headers.get('content-type')) === null || _a === void 0 ? void 0 : _a.includes('application/json'))
                    body = yield res.json();
                else
                    body = yield res.text();
                if (body && !body.success) {
                    const error = { code: body.error.code, message: body.error.message };
                    throw new ApiError_1.default({ path, status: res.status, error: error, method });
                }
                return body;
            }
            catch (error) {
                console.error(error);
                throw error;
            }
        });
    }
    get(path, { data = {}, headers = {} } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.build({ method: 'get', path, data, headers });
        });
    }
    post(path, { data = {}, headers = {} }) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.build({ method: 'post', path, data, headers });
        });
    }
    patch(path, { data = {}, headers = {} }) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.build({ method: 'patch', path, data, headers });
        });
    }
    delete(path, { data = {}, headers = {} } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.build({ method: 'delete', path, data, headers });
        });
    }
}
exports.default = Rest;
//# sourceMappingURL=index.js.map