"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Voice = void 0;
const events_1 = require("events");
const ws_1 = __importDefault(require("ws"));
const RTC_URL = 'wss://us-east1-rtc-staging.hiven.io';
class Voice extends events_1.EventEmitter {
    constructor(settings = { reconnectInt: 5000, reconnectCount: 5 }, cache = {}) {
        super();
        this.reconnectionCount = 0;
        this.settings = settings;
        this.cache = cache;
    }
    sendOp(op, opts) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (((_a = this.ws) === null || _a === void 0 ? void 0 : _a.readyState) != ws_1.default.OPEN)
                return;
            if (op == 2) {
                this.cache.token = opts === null || opts === void 0 ? void 0 : opts.data.token;
            }
            if (opts === null || opts === void 0 ? void 0 : opts.data)
                return this.ws.send(JSON.stringify({ op, e: opts === null || opts === void 0 ? void 0 : opts.e, d: opts === null || opts === void 0 ? void 0 : opts.data }));
            return this.ws.send(JSON.stringify({ op, e: opts === null || opts === void 0 ? void 0 : opts.e }));
        });
    }
    startHeartbeat(int) {
        return __awaiter(this, void 0, void 0, function* () {
            this.heartbeatInterval = setInterval(() => __awaiter(this, void 0, void 0, function* () { return this.sendOp(8); }), int);
        });
    }
    destroy() {
        var _a, _b;
        (_a = this.ws) === null || _a === void 0 ? void 0 : _a.removeAllListeners();
        (_b = this.ws) === null || _b === void 0 ? void 0 : _b.close();
        if (this.heartbeatInterval)
            clearInterval(this.heartbeatInterval);
    }
    init(room_id, user_id, join_token) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                this.ws = new ws_1.default(`${RTC_URL}`);
                this.ws.on('open', () => __awaiter(this, void 0, void 0, function* () {
                    this.reconnectionCount = 0;
                    this.sendOp(0, {
                        data: {
                            room_id,
                            user_id,
                            join_token
                        }
                    });
                    yield this.startHeartbeat(20000);
                    return resolve(true);
                }));
                this.ws.on('close', () => __awaiter(this, void 0, void 0, function* () {
                    // Websocket closed, stop the heartbeat ping and attempt reconnect using the settings from the client
                    if (process.env.DEBUG)
                        console.log(`Disconnected, waiting ${this.settings.reconnectInt}ms before reconnecting`);
                    setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                        this.destroy();
                        yield this.reconnect();
                    }), this.settings.reconnectInt);
                }));
                this.ws.on('message', (data) => __awaiter(this, void 0, void 0, function* () {
                    var _a, _b, _c, _d;
                    try {
                        const body = JSON.parse(data);
                        switch (body.op) {
                            case 2:
                                switch (body.e) {
                                    case 'ROUTER_RTP_CAPABILITIES':
                                        this.codecs = body.d.codecs;
                                        this.headerExtensions = body.d.headerExtensions;
                                        break;
                                    case 'RTC_TRANSPORTS':
                                        this.RTCTransports = body.d;
                                        (_a = this.RTCTransports) === null || _a === void 0 ? void 0 : _a.recv.dtlsParameters.fingerprints;
                                        this.sendOp(1, {
                                            e: 'SUBSCRIBE',
                                            data: {
                                                rtpCapabilities: {
                                                    codecs: this.codecs,
                                                    headerExtensions: this.headerExtensions
                                                }
                                            }
                                        });
                                        this.sendOp(1, {
                                            e: 'TRANSPORT_DTLS_CONNECT',
                                            data: {
                                                id: (_b = this.RTCTransports) === null || _b === void 0 ? void 0 : _b.recv.id,
                                                dtlsParameters: {
                                                    role: 'client',
                                                    fingerprints: (_c = this.RTCTransports) === null || _c === void 0 ? void 0 : _c.recv.dtlsParameters.fingerprints
                                                }
                                            }
                                        });
                                        this.sendOp(1, {
                                            e: 'CONSUMER_CREATE',
                                            data: {
                                                transportId: (_d = this.RTCTransports) === null || _d === void 0 ? void 0 : _d.send.id
                                            }
                                        });
                                        this.emit('connection', this.RTCTransports);
                                        break;
                                    default:
                                        break;
                                }
                                //  ROUTER_RTP_CAPABILITIES
                                break;
                            default:
                                break;
                        }
                        this.emit('data', body);
                    }
                    catch (error) { } /* eslint-disable-line no-empty */
                }));
            });
        });
    }
    reconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init(this.cache.room_id, this.cache.user_id, this.cache.join_token);
            // (TODO: THIS IS NOT THE PLACE FOR THIS) Login again using the cached credentials
            // await this.sendOp(2, { token: this.cache.token });
            return true;
        });
    }
}
exports.Voice = Voice;
//# sourceMappingURL=index.js.map