"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WS = void 0;
// Modules
const events_1 = require("events");
const ws_1 = __importDefault(require("ws"));
// Static variables
const ENCODING = 'json';
const COMPRESSION = 'text';
const SWARM_URL = `${process.env.SWARM_HOST || 'wss://swarm.hiven.io'}/socket?compression=${COMPRESSION}_json&encoding=${ENCODING}`;
// WS Class
class WS extends events_1.EventEmitter {
    constructor(settings = { reconnectInt: 5000, reconnectCount: 5 }, cache = {}) {
        super();
        this.reconnectionCount = 0;
        this.settings = settings;
        this.cache = cache;
    }
    sendOp(op, data) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (((_a = this.ws) === null || _a === void 0 ? void 0 : _a.readyState) != ws_1.default.OPEN)
                return false;
            if (op == 2) {
                this.cache.token = data.token;
            }
            if (data)
                return this.ws.send(JSON.stringify({ op, d: data }));
            return this.ws.send(JSON.stringify({ op }));
        });
    }
    startHeartbeat(int) {
        return __awaiter(this, void 0, void 0, function* () {
            this.heartbeatInterval = setInterval(() => __awaiter(this, void 0, void 0, function* () { return this.sendOp(3); }), int);
        });
    }
    destroy() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            yield ((_a = this.ws) === null || _a === void 0 ? void 0 : _a.removeAllListeners());
            yield ((_b = this.ws) === null || _b === void 0 ? void 0 : _b.close());
            if (this.heartbeatInterval)
                clearInterval(this.heartbeatInterval);
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                this.ws = new ws_1.default(`${SWARM_URL}`);
                this.ws.on('open', () => __awaiter(this, void 0, void 0, function* () {
                    this.reconnectionCount = 0;
                    return resolve(true);
                }));
                this.ws.on('close', () => __awaiter(this, void 0, void 0, function* () {
                    // Websocket closed, stop the heartbeat ping and attempt reconnect using the settings from the client
                    if (process.env.DEBUG)
                        console.log(`Disconnected, waiting ${this.settings.reconnectInt}ms before reconnecting`);
                    setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                        var _a;
                        yield ((_a = this.ws) === null || _a === void 0 ? void 0 : _a.removeAllListeners());
                        yield this.reconnect();
                    }), this.settings.reconnectInt);
                }));
                this.ws.on('message', (data) => __awaiter(this, void 0, void 0, function* () {
                    try {
                        const body = JSON.parse(data);
                        // Detect if connnection packet
                        if (body.op == 1 && body.d && body.d.hbt_int) {
                            // This is the heartbeat ping, starting int
                            yield this.startHeartbeat(body.d.hbt_int);
                        }
                        this.emit('data', body);
                    }
                    catch (error) { } /* eslint-disable-line no-empty */
                }));
            });
        });
    }
    reconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            // (TODO: THIS IS NOT THE PLACE FOR THIS) Login again using the cached credentials
            yield this.sendOp(2, { token: this.cache.token });
            return true;
        });
    }
}
exports.WS = WS;
//# sourceMappingURL=index.js.map